#ifndef EXERCISE_3_ERRORMEASUREMENTS_H_
#define EXERCISE_3_ERRORMEASUREMENTS_H_

/** Error measurements in two different norms
 *
 *  \file errorMeasurements.h
 */

#include <discreteFunctionShellFE.h>
#include <unityTriangleIntegratorShellFE.h>

//! \brief Calculates the \f$L^2\f$ norm of the difference between exact and numerical solution
template< typename ConfiguratorType >
class L2NormSqrDiff :
        public shellFE::UnitTriangleFENonlinIntegrationScalarIntegratorShellFE <ConfiguratorType, L2NormSqrDiff <ConfiguratorType> >
{
protected:
    typedef typename ConfiguratorType::RealType RealType;
    typedef typename ConfiguratorType::VectorType VectorType;
    typedef typename ConfiguratorType::TangentVecType TangentVecType;

    static const int dim = 3;

    std::unique_ptr<typename ConfiguratorType::VectorType> _dof;
    std::unique_ptr<shellFE::DiscreteVectorFunctionDefaultShellFE<ConfiguratorType>> _x;

    const shellFE::DiscreteFunctionDefaultShellFE<ConfiguratorType> _numericalSolution;

public:
    L2NormSqrDiff(const ConfiguratorType &conf, const VectorType &numericalSolution)
: shellFE::UnitTriangleFENonlinIntegrationScalarIntegratorShellFE <ConfiguratorType, L2NormSqrDiff <ConfiguratorType> > (conf),
  _numericalSolution(this->_config, numericalSolution)
  {
        int numGlobalDofs = this->_config.getNumGlobalDofs();
        const typename ConfiguratorType::InitType &grid = this->_config.getInitializer();
        _dof.reset(new typename ConfiguratorType::VectorType (dim * numGlobalDofs));

        for (int i = 0; i < numGlobalDofs; ++i) {
            for (int c = 0; c < dim; ++c) {
                (*_dof)[c*numGlobalDofs + i] = grid.getVertex(i)[c];
            }
        }

        _x.reset(new shellFE::DiscreteVectorFunctionDefaultShellFE<ConfiguratorType>(this->_config, *_dof, dim));
  }

    RealType evaluateIntegrand ( const typename ConfiguratorType::ElementType &el, int q) const
    {
        RealType nl = 0.0;

        //! \todo Evaluate \f$ L^2 \f$ norm

        return nl;
    }
};

//! \brief Calculates the \f$H^1\f$ norm of the difference between exact and numerical solution
template<typename ConfiguratorType>
class H1NormSqrDiff :
        public shellFE::UnitTriangleFENonlinIntegrationScalarIntegratorShellFE <ConfiguratorType, H1NormSqrDiff <ConfiguratorType> >
{
protected:
    typedef typename ConfiguratorType::RealType RealType;
    typedef typename ConfiguratorType::TangentVecType TangentVecType;
    typedef typename ConfiguratorType::VectorType VectorType;
    typedef typename ConfiguratorType::DomVecType DomVecType;

    static const int dim = 3;

    std::unique_ptr<typename ConfiguratorType::VectorType> _dof;
    std::unique_ptr<shellFE::DiscreteVectorFunctionDefaultShellFE<ConfiguratorType>> _x;

    const shellFE::DiscreteFunctionDefaultShellFE<ConfiguratorType> _numericalSolution;

public:
    H1NormSqrDiff(const ConfiguratorType &conf, const VectorType &numericalSolution)
: shellFE::UnitTriangleFENonlinIntegrationScalarIntegratorShellFE <ConfiguratorType, H1NormSqrDiff <ConfiguratorType> > (conf),
  _numericalSolution(this->_config, numericalSolution)
  {
        int numGlobalDofs = this->_config.getNumGlobalDofs();
        const typename ConfiguratorType::InitType &grid = this->_config.getInitializer();
        _dof.reset(new typename ConfiguratorType::VectorType (dim * numGlobalDofs));

        for (int i = 0; i < numGlobalDofs; ++i) {
            for (int c = 0; c < dim; ++c) {
                (*_dof)[c*numGlobalDofs + i] = grid.getVertex(i)[c];
            }
        }

        _x.reset(new shellFE::DiscreteVectorFunctionDefaultShellFE<ConfiguratorType>(this->_config, *_dof, dim));
  }

    RealType evaluateIntegrand ( const typename ConfiguratorType::ElementType &el, int q) const
    {
        RealType nl = 0.0;

        //! \todo Evaluate \f$ H^1 \f$ norm

        return nl;
    }
};

#endif /* EXERCISE_3_ERRORMEASUREMENTS_H_ */
